/* eslint-disable */
import { deepClone } from "../utils/util"

const DEFAULT_PAGE = 1
const DataTableMixin = {
  data() {
    return {
      _url: '',
      _params: {},
      _sort: {}, // 排序
      query: {},
      page: DEFAULT_PAGE,
      pageSize: 10,
      loading: false, // 是否在加载数据
      list: [], // 数据列表
      total: 0
    }
  },
  methods: {
    objectKeyIsEmpty(obj) {
      for (const key in obj) {
        if (obj.hasOwnProperty(key)) {
          if (obj[key] === null || obj[key] === '') {
            delete obj[key]
          }
        }
      }
      return obj
    },
    setDataTableUrl(url) {
      this._url = url
    },
    /**
     * 拉取数据
     * @returns {Promise<any>}
     */
    async dataTableInit() {
      if (!await this.dataTableBeforeInit()) {
        return
      }
      return new Promise((resolve, reject) => {
        this.loading = true
        let request = null
        // 判断是否有挂载$http， 或使用Sr组件
        const http = this.$http || this.$ldpOptions?.http
        if (http && http.get) {
          request = http.get(this._url, {
            params: this.objectKeyIsEmpty(this._params)
          })
        }
        request.then(res => {
          const data = res.headers && res.request ?  res.data : res
          if (data && data.pageData instanceof Array) {
            this.list = data.pageData
            this.total = data.totalCount || data.total
          } else if (data.data && data.data.pageData instanceof Array) {
            this.list = data.data.pageData
            this.total = data.data.totalCount || data.total
          } else if (data.data instanceof Array && data.data.length) {
            this.list = data.data
            this.total = data.data.totalCount || data.total
          } else {
            this.list = []
          }
          setTimeout(() => {
            this.loading = false
          }, 170)
          resolve(res)
        }).catch(err => {
          this.loading = false
          reject(err)
        })
      })
    },

    /**
     * 初始化参数
     * @returns {boolean}
     */
    dataTableBeforeInit() {
      let query = deepClone(this.query)
      // 判断是否有搜索ref， 如果有则默认用搜索组件的数据
      if (this.$refs.search) {
        query = this.$refs.search.getData()
      }
      this._params = Object.assign({
        pageSize: this.pageSize,
        pageIndex: this.page,
        ...this._sort
      }, query)
      return true
    },

    /**
     * 翻页
     * @param val
     */
    dataTablePageChange(val) {
      this.page = val
      this.dataTableInit()
    },

    /**
     * 改变每页大小
     * @param val
     */
    dataTableSizeChange(val) {
      this.page = DEFAULT_PAGE
      this.pageSize = val
      this.dataTableInit()
    },
    /**
     * 排序改变
     */
    dataTableSortChange(sort = {}) {
      if (sort) {
        this._sort = sort
        this.page = DEFAULT_PAGE
        this.dataTableInit()
      }
    },
    /**
     * 搜索
     * @param {object} val 查询的数据
     */
    dataTableSearch(val) {
      if (val) {
        this.query = val
      }
      this.page = DEFAULT_PAGE
      this.total = 0
      // this.list = [] // 数据列表
      this.dataTableInit()
    },
    /**
     * 预防删除第二页最后一条数据时，或者多选删除第二页的数据时，页码错误导致请求无数据
     * @param {number} delAcount 被删除的行数， 默认为1
     */
    dataTableDelChangePage(delAcount = 1) {
      if (this.list.length === delAcount && this.page !== DEFAULT_PAGE) {
        this.page = this.page - 1
      }
    },
    tableFilterList(data) {
      if (data instanceof Array) {
        const arr = []
        const res = []
        data.forEach(i => {
          if (i.query) {
            if (i.query.type === 'group-input') {
              const obj = {
                label: i.label,
                value: i.property || i.prop
              }
              arr.push(obj)
            } else if (i.query.type === 'databox') {
              const obj = {
                type: i.query.type,
                label: i.label,
                key: i.property || i.prop,
                ...i.query
              }
              if (i.query.hidden !== undefined) {
                Object.assign(obj, { hidden: i.query.hidden })
              }
              res.push(obj)
            } else {
              const obj = {
                ...i.query,
                type: i.query.type,
                label: i.label,
                key: i.property || i.prop,
                data: i.query.data,
                value: i.query.value
              }
              if (i.query.hidden !== undefined) {
                Object.assign(obj, { hidden: i.query.hidden })
              }
              res.push(obj)
            }
          }
        })
        if (arr.length) {
          const obj = {
            type: 'group-input',
            key: 'groupInput',
            data: arr
          }
          res.push(obj)
        }
        return res
      }
      return data
    }
  }
}

export default DataTableMixin
