import { hasOwnProperty, warn, parseUrl, uuid } from '@/utils/util'
import FormDataFormat from '@/utils/formDataFormat'
import { getRequestConfig, isSuccess } from '@/utils/request'
export default {
  props: {
    submit: Object,
    save: Object,
    // 显示提交按钮
    showSubmit: Boolean,
    // 显示保存按钮
    showSave: Boolean,
    // 显示重置按钮
    showReset: Boolean,
    // 显示取消按钮
    showCancel: Boolean,
    // 显示搜索按钮
    showSearch: Boolean,
    // 显示按钮的排序
    btnOrder: Array,
    // 显示默认按钮。 根据当前mode来判断显示
    showBtns: {
      type: Boolean,
      default: true
    },
    showSuccessMsg: {
      type: Boolean,
      default: true
    },
    // 自定义按钮
    buttons: [Array, Object],
    // 按钮样式
    buttonsStyle: Object,

    // 用于格式化表单
    formatData: Function,
    // http请求对象
    http: Object
  },
  data() {
    return {
      formDataFormat: new FormDataFormat(this.$ldpOptions)
    }
  },
  computed: {
    defaultBtns() {
      return {
        search: {
          name: this.$attrs.searchName || '查询',
          type: 'primary',
          icon: 'el-icon-search',
          click: this.doSearch
        },
        submit: {
          name: this.$attrs.submitName || '提交',
          type: 'primary',
          loading: false,
          click: this.doSubmit
        },
        save: {
          name: this.$attrs.saveName || '保存',
          type: 'primary',
          loading: false,
          click: this.doSave
        },
        cancel: {
          name: this.$attrs.cancelName || '取消',
          click: this.doCancel
        },
        reset: {
          name: this.$attrs.resetName || '重置',
          icon: 'el-icon-refresh-left',
          click: this.doReset
        }
      }
    },
    btns() {
      // const defaultBtns = this.defaultBtns
      if (this.buttons) {
        return this.buttons.fliter(btn => {
          // 通过match来过滤 是否此按钮支持当前表单模式，如新增一个按钮，编辑一个按钮。
          if (btn.match && btn.match.includes(this.mode)) {
            return btn
          }
        })
      }

      const btns = []
      if (this.showSubmit) {
        btns.push(this.defaultBtns.submit)
      }
      if (this.showSave) {
        btns.push(this.defaultBtns.save)
      }
      if (this.showSearch) {
        btns.push(this.defaultBtns.search)
      }
      if (this.showReset) {
        btns.push(this.defaultBtns.reset)
      }
      if (this.showCancel) {
        btns.push(this.defaultBtns.cancel)
      }
      if (btns.length > 0) return btns

      if (!this.showBtns) return []
    }
  },
  methods: {
    // 按钮配置对象的事件
    doEvent(event, btn) {
      if (['reset', 'onReset'].includes(event)) {
        this.doReset()
      } else if (['search', 'onSearch'].includes(event)) {
        this.doSearch()
      } else if (['cancel', 'onCancel'].includes(event)) {
        this.doCancel()
      } else if (['submit', 'onSubmit'].includes(event)) {
        this.doSubmit(btn)
      } else if (['save', 'onSave'].includes(event)) {
        this.doSave(btn)
      } else if (['link', 'onLink'].includes(event) && btn.url) {
        this.$router.push(btn.url)
      } else if (['bpm', 'onBpm'].includes(event)) {
        const bpmstatus = this.formData[this.bpmstatusField]
        this.$refs.bpm.open(this.getFormSubmitData(), bpmstatus)
      } else if (['onRefernceFunc'].includes(event)) {
        if (btn.eventType === 'attachmentTable') {
          if (!this.id) {
            const id = uuid()
            this.$emit('update:id', id)
            this.formData.id = id
            this.$refs.attachmentTable.open({
              id
            })
          } else {
            this.$refs.attachmentTable.open()
          }
        }
      } else if (btn.click) {
        btn.click()
      } else {
        this.$emit('button-click', btn)
      }
    },
    doClick(btn) {
      const click = () => {
        if (btn.action) {
          // 判断是否有默认动作， 如果是则执行，默认动作
          if (this.defaultBtns[btn.action]) {
            this.defaultBtns[btn.action](btn)
            return
          }
        }
        btn.click && btn.click(btn)
      }
      if (btn.before) {
        if (btn.before()) {
          return click(btn)
        }
      } else {
        return click(btn)
      }
    },
    /**
     * 点击默认搜索按钮
     */
    doSearch() {
      this.$emit('search', this.formData)
    },
    /**
     * 点击重置按钮
     */
    doReset() {
      this.initFormData()
      this.$emit('reset', this.formData)
    },
    async doSubmit(btn) {
      if (btn.loading) return
      this.$set(btn, 'loading', true)
      try {
        // 校验数据是否通过
        await this.validate()
        // 获取需要提交的数据对象
        const data = this.getFormSubmitData()
        // 获取接口
        const request = this.getDefaultRequest(btn, 'submit')
        const config = getRequestConfig(request, data)
        this.$emit('submit-click', data, this.priItems)
        if (config) {
          this.$emit('submit-request', data, this.priItems)
          this.$http(config)
            .then(res => {
              // 接口请求成功后延迟200毫秒
              setTimeout(() => {
                btn.loading = false
              }, 200)
              // 判断是否有完成处理， 如果有则执行。
              if (btn.finish) {
                if (btn.finish(res)) return
              }
              // 判断是否成功
              if (typeof this.$ldpOptions.isRequestSuccess === 'function') {
                if (!this.$ldpOptions.isRequestSuccess(res)) {
                  // 如果错误则提示
                  this.$emit('save-error', res)
                  this.requestError(res)
                  return
                }
              } else {
                const success = this.$ldpOptions?.response?.success
                if (success) {
                  if (!res[success]) {
                    this.$emit('save-error', res)
                    this.requestError()
                    return
                  }
                }
                const code = this.$ldpOptions?.response?.code
                if (typeof code === 'object' && code.name && code.value) {
                  if (res[code.name] !== code.value) {
                    this.$emit('save-error', res)
                    this.requestError()
                    return
                  }
                }
                if (!res.code || Number(res.code) !== 200) {
                  this.$emit('save-error', res)
                  this.requestError()
                  return
                }
              }
              const dataField = this.$ldpOptions?.response?.data || 'data'
              this.$emit('submit-success', res[dataField])
              // 判断是否提示
              if (this.showSuccessMsg) {
                const message = this.$ldpOptions?.response?.message
                this.$message.success(res[message] || '操作成功')
              }

              // 判断执行默认操作。
              if (this.successDefaultAction) {
                this.doCancel()
              }
              return res
            })
            .catch(err => {
              btn.loading = false
              // 判断是否有错误处理，如果有则执行
              if (btn.error) {
                btn.error(err)
              }
            })
          return
        }
        btn.loading = false
      } catch (error) {
        btn.loading = false
      }
      this.$emit('submit')
    },
    doBpm(btn) {
      if (btn.event === 'bpm') {
        this.$refs.bpm.open(this.getFormSubmitData())
        return
      }
      const url = parseUrl(btn.url, this.formData)
      if (url) {
        this.$prompt('请输入审批意见', '提示', {
          confirmButtonText: '确定',
          cancelButtonText: '取消'
        }).then(({ value }) => {
          const data = this.getFormSubmitData()
          this.$http({
            method: 'post',
            url: url,
            params: { comment: value },
            data
          })
            .then(res => {
              if (isSuccess(res, this.$ldpOptions?.response)) {
                this.$message.success(btn.message || '操作成功')
                if (this.back !== false) {
                  this.$router.back()
                }
              }
            })
        }).catch(() => {
        })
      }
    },
    /**
     * 点击默认提交
     */
    async doSave(btn) {
      if (btn.loading) return
      this.$set(btn, 'loading', true)
      try {
        // 校验数据是否通过
        await this.validate()
        // 获取需要提交的数据对象
        const data = this.getFormSubmitData()
        // 获取接口
        const request = this.getDefaultRequest(btn)
        const config = getRequestConfig(request, data)
        this.$emit('save', data, this.priItems)
        if (config) {
          this.$http(config)
            .then(res => {
              // 接口请求成功后延迟200毫秒
              setTimeout(() => {
                btn.loading = false
              }, 200)
              // 判断是否有完成处理， 如果有则执行。
              if (btn.finish) {
                if (btn.finish(res)) return
              }
              // 判断是否成功
              if (typeof this.$ldpOptions.isRequestSuccess === 'function') {
                if (!this.$ldpOptions.isRequestSuccess(res)) {
                  // 如果错误则提示
                  this.$emit('save-error', res)
                  this.requestError(res)
                  return
                }
              } else {
                const success = this.$ldpOptions?.response?.success
                if (success) {
                  if (!res[success]) {
                    this.$emit('save-error', res)
                    this.requestError()
                    return
                  }
                }
                const code = this.$ldpOptions?.response?.code
                if (typeof code === 'object' && code.name && code.value) {
                  if (res[code.name] !== code.value) {
                    this.$emit('save-error', res)
                    this.requestError()
                    return
                  }
                }
                if (!res.code || Number(res.code) !== 200) {
                  this.$emit('save-error', res)
                  this.requestError()
                  return
                }
              }
              const dataField = this.$ldpOptions?.response?.data || 'data'
              this.$emit('save-success', res[dataField])
              // 判断是否提示
              if (this.showSuccessMsg) {
                const message = this.$ldpOptions?.response?.message
                this.$message.success(res[message] || '操作成功')
              }

              // 判断执行默认操作。
              if (this.successDefaultAction) {
                this.doCancel()
              }
              return res
            })
            .catch(err => {
              btn.loading = false
              // 判断是否有错误处理，如果有则执行
              if (btn.error) {
                btn.error(err)
              }
            })
          return
        }
        btn.loading = false
      } catch (error) {
        btn.loading = false
      }
    },
    /**
     * 获取默认提交请求信息
     */
    getDefaultRequest(btn) {
      // 根据当前模式获取到对应的提交请求信息
      let request = null
      if (btn && btn.url) return btn.url
      let submit = btn.request
      const event = btn.event
      if (event === 'save') {
        return btn.request || this.save
      }
      if (!submit) {
        submit = event ? this[event] : this.submit
      }
      if (this.mode === 'edit' && submit?.edit) {
        request = submit.edit?.request
          ? submit.edit.request
          : submit.edit
      } else if (this.mode === 'add' && submit?.add) {
        request = submit.add?.request
          ? submit.add.request
          : submit.add
      } else if (submit?.request) {
        request = submit.request
      } else if (submit?.url) {
        request = submit
      }
      return request
    },
    /**
     * 请求失败错误处理
     */
    requestError(res) {
      // 判断是否显示错误提示
      if (this.showErrorMsg && res?.message) {
        this.$message.error(res.message)
      }
      // this.$emit('request-error', res)
    },
    doCancel() {
      this.$emit('cancel')
    },
    // 校验数据格式
    validate() {
      const form = this.getFormRef()
      return form.validate()
    },
    /**
     * 获取表单提交数据
     */
    getFormSubmitData() {
      const data = this.formatData ? this.formatData(this.formData, this.priItems) : this.formDataFormat.getFormSubmitData(this.formData, this._priItems)
      if (this.id && !data.id) {
        data.id = this.id
      }
      return data
    },
    /**
     * 设置日期时间范围的数据
     */
    setRangeData(item, formData, data) {
      if (
        ['time', 'date', 'datetime'].includes(item.type) &&
        item?.props?.isRange
      ) {
        const value = formData[item.key]
        if (Array.isArray(value)) {
          if (item.rangeKeys) {
            item.rangeKeys.forEach((i, index) => {
              data[i] = value[index]
            })
            return true
          }
        }
      }
    },
    /**
     * 通过数据格式化获取实际提交数据
     */
    getDataByDataFormat(item, formData) {
      let dataFormat = item.dataFormat
      if (!dataFormat) {
        if (this.$ldpOptions?.dataFormat) {
          // 判断是否同类型，如果同类型则自动转换格式
          if (hasOwnProperty(this.$ldpOptions.dataFormat, item.type)) {
            dataFormat = this.$ldpOptions.dataFormat[item.type]
          }
        }
      }
      if (!dataFormat) return null
      const value = formData[item.key]
      if (typeof dataFormat === 'function') {
        return dataFormat(value, formData, item)
      }
      if (value === null || value === undefined) return null
      if (typeof dataFormat === 'string') {
        // 判断是否是分割， 默认分割为 ，
        if (dataFormat === 'split' && Array.isArray(value)) {
          return this.formatSplit(value)
        } else if (['number', Number].includes(dataFormat)) {
          if (typeof value === 'boolean') return value ? 1 : 0
          if (typeof value === 'object') {
            warn(`[ldp-form waring] ${item.key} dataFormat type error`)
            return value
          }
          return Number(value)
        } else if (dataFormat === 'json') {
          return JSON.stringify(value)
        } else if (['object', Object].includes(dataFormat)) {
          // 保存为对象类型，先判断当前类型是否为对象， 如果是则直接保存，如果不是则自动封装成对象， 默认key为id
          if (typeof value === 'object') return value
          return {
            id: value
          }
        }
      } else if (typeof dataFormat === 'object') {
        if (dataFormat.split) {
          return this.formatSplit(value, dataFormat.split)
        } else if (dataFormat.filterFields) {
          if (typeof value === 'object') {
            const data = {}
            dataFormat.filterFields.forEach(i => {
              data[i] = value[i]
            })
            return data
          }
          warn(
            `[ldp-form waring] ${item.key} dataFormat   value is not obejct`
          )
        }
      }
      return null
    },
    formatSplit(value, formatStr = ',') {
      // 判断是否是对象，如果成员是对象，则默认从id获取。
      const list = value.map(v => {
        if (typeof v === 'object' && v !== null) {
          if (v.id) return v.id
        }
        return v
      })
      return list.join(formatStr)
    },
    // 设置模式为编辑
    setMode(mode) {
      this.$emit('update:mode', mode)
    }
  }
}
