import { deepClone } from '@/utils/util'
export default class FormStore {
  constructor(form, data) {
    this.form = form
    this.formItems = data
  }
  setInitItems(data) {
    this.initItems = deepClone(data)
  }
  resetItems() {
    this.formItems = deepClone(this.initItems)
  }
  setFormItems(data) {
    this.formItems = data
  }
  setItemProperty(key, prop, value) {
    const item = this.getItem(key)
    if (item) {
      let tmp = item
      let _prop = prop
      if (prop.indexOf('.') !== -1) {
        const ary = prop.split('.')
        for (let i = 0; i < ary.length - 1; i++) {
          tmp = tmp[ary[i]]
          _prop = ary[i]
        }
      }
      this.form.$set(tmp, _prop, value)
      return item
    }
    return false
  }
  /**
   *
   * @param {*} item 待添加的表单项
   * @param {*} index 添加到父节点的序号，默认为最后
   * @param {*} parent 待添加的父节点
   */
  addItem(item, index, parent = this.formItems) {
    if (typeof parent === 'string') {
      const info = this.findItem(parent)
      if (!info) {
        return false
      }
    }
    if (index === undefined) {
      parent.push(item)
    } else {
      parent.splice(index, 0, item)
    }
    return true
  }
  replaceItem(item, key) {
    const info = this.findItem(this.formItems, key)
    if (info) {
      info.parent.splice(info.index, 1, item)
    }
  }
  /**
   * 获取表单项
   * @param {*} key 表单项key
   * @returns
   */
  getItem(key) {
    const info = this.findItem(key, this.formItems)
    if (info) return info.item
    return null
  }
  /**
   *  移除当前表单项
   * @param {*} key 表单的key
   * @returns
   */
  removeItem(key) {
    const info = this.findItem(key)
    if (!info) return false
    info.parent.splice(info.index, 1)
    return true
  }

  findItem(key, parent = this.formItems) {
    for (const index in parent) {
      const item = parent[index]
      if (item.key === key) {
        return {
          parent,
          item,
          index
        }
      } else if (item.children) {
        const data = this.findItem(key, item.children)
        if (data) return data
      }
    }
  }
}
