
import { deepClone, warn } from '@/utils/util'
/**
 * 初始化表单数据
 */
export function initFormData(items, data, mode) {
  const form = {}
  items.forEach(item => {
    const key = item.key
    if (!key) {
      warn('[ldp-form] key 不能为空')
      return
    } else if (item.type === 'container' && item.children) {
      initFormData(item.children, data, mode)
    } else {
      if (data && item.key in data && data[item.key] !== undefined) {
        form[key] = data[item.key]
      } else {
        form[key] = initFormItemData(item, mode)
      }
    }
  })
  return form
}

/**
 * 初始化表单项数据
 */
export function initFormItemData(item, mode, isDefualt = true) {
  let value
  // 判断是否为编辑模式， 如果是编辑模式则不设置默认值
  if (mode && mode === 'edit') {
    if (item.type === 'select' && item.multiple) {
      return []
    } else if (item.type === 'checkbox-group' || item.type === 'checkboxGroup') {
      return []
    }
    return ''
  }
  if (isDefualt) {
    // 判断是否默认值是funciton，则直接调用，并获取返回值做为默认值
    if (typeof item.defaultValue === 'function') {
      value = item.defaultValue()
    } else if (item.type === 'input') {
      value = item.defaultValue
    } else if (item.type === 'select') {
      return initSelect(item)
    } else if (item.type === 'time') {
      value = item.defaultValue
    } else if (item.type === 'checkbox-group' || item.type === 'checkboxGroup') {
      value = item.defaultValue || []
    } else if (item.type === 'checkbox') {
      value = item.defaultValue || false
    } else {
      value = item.defaultValue
    }
  } else if (item.type === 'checkbox-group' || item.type === 'checkboxGroup') {
    value = []
  } else if (item.type === 'select' && item.multiple) {
    value = []
  } else if (item.type === 'checkbox') {
    value = false
  }
  return value
}

/**
 * 初始化Select 的默认值
 * @param {object} item
 * @param {string, array} value
 * @returns
 */
function initSelect(item, value) {
  const defaultValue = value || item.defaultValue
  const multiple = item.multiple || item.props?.multiple
  // 判断是否是字符串， 如果多选select 且为字符串则需要使用分割。 默认分隔符，
  if (multiple) {
    // 判断是否是
    if (typeof defaultValue === 'string') {
      return defaultValue.split(',')
    } else if (Array.isArray(defaultValue)) {
      return deepClone(defaultValue)
    }
    return []
  } else if (typeof defaultValue !== 'undefined') {
    return defaultValue
  }
  return
}
