import qs from 'qs'
import { deepClone, parseUrl, getFieldData } from './util'
export function getRequestConfig(request, data) {
  let tmp = deepClone(data)
  let url = typeof request === 'object' && request ? request.url : request
  if (!url) return null
  // 判断格式，是否是formdata 如果是则转化成form-data
  let type = 'body'
  if (typeof request === 'object') {
    if (request.contentType === 'multipart/form-data') {
      const formData = new FormData()
      Object.keys(data).forEach(i => {
        formData.append(i, data[i])
      })
      tmp = formData
    } else if (request.contentType === 'application/x-www-form-urlencoded') {
      tmp = qs.stringify(data)
    }
    type = request.type
  }
  url = parseUrl(url, data)
  const config = {
    url: url,
    method: request.method || 'post',
    headers: {
      'Content-Type': request.contentType || 'application/json'
    }
  }
  if (type === 'query') {
    config.params = tmp
    delete config.data
  } else {
    config.data = tmp
  }
  return config
}

/**
 * 获取返回的实际数据
 * @param {*} res 接口返回的数据
 * @param {*} responseOption 返回数据选项
 * @returns
 */
export function getResponseData(res, responseOption) {
  if (!responseOption) {
    return res || {}
  }

  const successField = responseOption?.successField || 'success'
  const dataField = responseOption?.dataField || 'data'
  const messageField = responseOption?.dataField || 'message'
  if (!isSuccess(res, successField)) {
    return {
      messageField: getFieldData(res, messageField),
      success: false
    }
  }
  return {
    success: true,
    data: getFieldData(res, dataField)
  }
}

/**
 * 判断接口请求返回数据是否成功， 默认判断success 或者code == 200
 * @param {*} res
 * @param {*} successField
 */
export function isSuccess(res, responseOption) {
  const successField = responseOption?.successField || 'success'
  if (!successField) {
    if (res.success || Number(res.code) === 200) {
      return true
    }
    return false
  }
  return getFieldData(res, successField)
}
