const path = require('path') // 引入path模块
function resolve(dir) {
  return path.join(__dirname, dir) // path.join(__dirname)设置绝对路径
}
module.exports = {
  publicPath: './',
  outputDir: 'dist',
  assetsDir: 'static',
  lintOnSave: false,
  productionSourceMap: false,
  pages: {
    index: {
      // page 的入口
      entry: 'example/main.js',
      // 模板来源
      template: 'public/index.html',
      // 在 dist/index.html 的输出
      filename: 'index.html'
    }
  },
  devServer: {
    historyApiFallback: true,
    noInfo: true,
    overlay: true,
    hot: true,
    open: true,
    proxy: {
      '/api/example-service': {
        // target: 'http://47.103.46.246:9080',
        target: 'http://192.168.0.137:8800',
        changeOrigin: true,
        pathRewrite: {
          '^/api/example-service': ''
        }
      },
      '/api/demo-service': {
        // target: 'http://47.103.46.246:9080',
        target: 'http://47.103.46.246:9080',
        changeOrigin: true,
        pathRewrite: {
          '^/api': ''
        }
      },
      '/api/mcs-service': {
        target: 'http://47.103.46.246:9080',
        changeOrigin: true,
        pathRewrite: {
          '^/api/mcs-service': '/mcs-service'
        }
      },
      '/dev-api': {
        target: 'http://47.103.46.246:9080',
        changeOrigin: true,
        pathRewrite: {
          '^/api': ''
        }
      }
    },
    after: require('./example/mock/mock-server.js')
  },
  configureWebpack: {
    devtool: 'source-map',
    resolve: {
      alias: {
        '@': resolve('src'),
        vue$: 'vue/dist/vue.esm.js'
      },
      extensions: ['*', '.js', '.vue', '.json']
    }
  },
  chainWebpack(config) {
    config.module
      .rule('vue')
      .use('vue-loader')
      .loader('vue-loader')
      .tap(options => {
        options.compilerOptions.preserveWhitespace = true
        return options
      })
      .end()
  }
}
